//============================================================================
// Name        : main.cpp
// Author      : Ruediger Knoerig
// Version     :
// Copyright   : GPL
// Description : Hello World in C++, Ansi-style
//============================================================================

#include <iostream>
#include "cv.h"
#include "highgui.h"
#include <string>
#include <vector>
#include <cmath>
#include <complex>

#include "CommandLineParser.h"
#include <stdexcept>

using namespace std;
using namespace cv;

void printMatrix(const Mat &a)
{
  cout << "=[";
  for(int r=0;r<a.rows;r++)
    {
      cout<< "[";
      for(int c=0;c<a.cols*a.channels();c++)
	{
	  cout << a.at<float>(r,c) << ",";
	}
      cout<< "];";
    }
  cout << "];" << endl;
}

Mat readImage(const string &name)
{
  Mat gray = imread(name,0);
  if(!gray.data) throw (string("Could not read image ")+name+string("!"));
  Mat bild1;
  gray.convertTo(bild1,CV_32FC1,1.0/255.0);
  return bild1;
}

void overlayDenoise(Mat &Ag,Mat &Bg,bool doSeg,float level)
{
  Mat As,Bs,H;
  if(doSeg)
    {
      threshold(Ag,As,level,1.0,THRESH_BINARY);
      threshold(Bg,Bs,level,1.0,THRESH_BINARY);
    }
  else
    {
      As=Ag;
      Bs=Bg;
    }
  dft(As,As,DFT_COMPLEX_OUTPUT);
  dft(Bs,Bs,DFT_COMPLEX_OUTPUT);
  mulSpectrums(As,Bs,H,DFT_COMPLEX_OUTPUT,true);
  // whitening
  for(int r=0;r<H.rows;r++)
    {
      complex<float> *rv=H.ptr<complex<float> >(r);
      for(int c=0;c<(H.cols>>1);c++)
	{
	  float m=abs(rv[c]);
	  if(m>=1e-5)
	    {
	      rv[c]/=m;
	    }
	  else
	    {
	      rv[c]=0;
	    }
	}
    }
  if(doSeg)
    {
      dft(Ag,As,DFT_COMPLEX_OUTPUT);
      dft(Bg,Bs,DFT_COMPLEX_OUTPUT);
    }
  
  mulSpectrums(Bs,H,Bs,DFT_COMPLEX_OUTPUT);
  As=0.5*(As+Bs);
  dft(As,Ag,DFT_COMPLEX_OUTPUT+DFT_REAL_OUTPUT+DFT_INVERSE+DFT_SCALE);
}


int main(int argc,char **argv) {
  try {
    CommandLineParser parser("This program overlays shift-compensated star images. Usage: ProgName [Options] Image1 ... ImageN");
    parser.addParameter("o","Name of the JPEG output file");
    parser.addParameter("doSegm","Perform a value segmentation.");
    parser.addParameter("v","Verbose output");
    parser.addParameter("display","Display result");
    parser.addParameter("SegmLvl","Level for the value segmentation","0.5");
    parser.parse(argc,argv,true);
    string outputfile=parser.getArgument("o");
    bool segmentation=parser.isSet("doSegm");
    float level=parser.getDoubleArgument("SegmLvl");
    bool verbose=parser.isSet("v");
    list<string> inputfiles=parser.getRemainingArguments();

    if(inputfiles.empty()) throw string("No input files!");
    if(verbose)
      {
	cout << "Overlay ";
	for(list<string>::const_iterator it=inputfiles.begin();it != inputfiles.end();it++) cout << *it << " ";
	cout << " to output file " << outputfile << "." << endl;
	if(segmentation)
	  {
	    cout << "Performing value segmentation with a level of " << level << "." << endl; 
	  }
	else
	  {
	    cout << "Don't perform level segmentation." << endl;
	  }
      }
    list<string>::const_iterator name=inputfiles.begin();
    Mat bild1=readImage(*(name++));
    for(;name != inputfiles.end();name++)
      {
	Mat bild2=readImage(*name);
	overlayDenoise(bild1,bild2,segmentation,level);
      }
    
    if(parser.isSet("display"))
      {
	namedWindow("Anzeige");
	imshow("Anzeige",bild1);
	waitKey(0);
      }
    Mat result;
    bild1*=255;
    bild1.convertTo(result,CV_8UC1);
    vector<int> attribute(2);
    attribute[0] = CV_IMWRITE_JPEG_QUALITY;
    attribute[1] = 100;
    imwrite(outputfile,result, attribute);
    return 0;
  }
  catch(const char *msg)
    {
      throw string(msg);
    }
  catch(exception e)
    {
      throw string(e.what());
    }
  catch (string msg) {
    cerr << "An error occured! " << msg << endl;
    return -1;
  }
}

